; -*- fundamental -*-
; $Id$
;
; Simple option ROM for printer for ABC80; this is not authentic, but
; was done for the FPGA project.  This assumes the following hardware
; interface:
;
; - ABC-bus select code 60 decimal;
; OUT 0 - (OUT)  data
; OUT 3 - (C2)   write high bit on active (end of job when active -> 0)
; INP 1 - (STAT) bit 5 - sending data
;                bit 6 - sending break
;                bit 7 - flow control
;
; This is a crude mimic of the original ABC80 Centronics interface,
; probably good enough to run the standard ABC80 printer option rom,
; except I don't have a copy of it
;
; Baud rate = 25 MHz / (C3:C2 + 1)
;
selcode: equ 60

	org 0x7800

jptable:
	jp open				; OPEN
	jp open				; PREPARE
	jp close			; CLOSE
	jp nothere			; INPUT
	jp print			; PRINT
	jp nothere			; Block to buf
	jp nothere			; Block from buf
	jp nothere			; DELETE???
	jp nothere			; RENAME???

open:
	call select

	ld a,128+selcode
	out (3),a
	
	in a,(1)
	and a
	jr z,done

notready:
	rst 0x10
	defb 128+51			; Err 51 = enheten upptagen

done:
	ld a,e				; Restore select
	ld (0xFD3F),a
	out (1),a

	and a
	ret


close:
	call select

close_wait:
	in a,(1)
	and a
	jr nz,close_wait


	ld a,selcode
	out (3),a

	jr done


print:
	call select

print_loop:
	ld a,b
	or c
	jr z,done

print_wait:
	in a,(1)
	and a
	jr nz,print_wait

	ld a,(hl)
	out (0),a

	dec bc
	inc hl
	jr print_loop

	;
	; IMPORTANT: This uses register E to save the previous select code.
	; If register E is used, you must PUSH DE/POP DE or equivalent.
	;
select:
	ld a,(0xFD3F)			; Old select code
	ld e,a
	ld a,selcode			; Select code
	ld (0xFD3F),a
	out (1),a
	ret

nothere:
	rst 0x10
	defb 128+52			; Err 52 = ej till denna enhet

